from zope import interface

class IMapped(interface.Interface):    
    __mapper__ = interface.Attribute(
        """ORM mapper.""")

class IMapper(interface.Interface):
    """An ORM mapper for a particular specification."""



from interfaces import IMapper
from zope.interface.declarations import _normalizeargs

from uuid import uuid1
from random import randint

from ore.alchemist import Session

def uuid():
    """Return new unique id as string.

    Force first character between 'a' and 'z'.
    """
    
    return chr(randint(ord('a'), ord('z'))) + uuid1().hex[:-1]

def create(spec):
    # set up mapper
    mapper = IMapper(spec)

    # create instance
    instance = mapper()

    # set soup attributes
    instance.uuid = uuid()
    instance.spec = mapper.__spec__

    # save to session
    session = Session()
    session.save(instance)

    return instance



from zope import component

import sqlalchemy as rdb
from sqlalchemy import orm
        
from ore.alchemist.interfaces import IDatabaseEngine

from interfaces import IMapped

import relations

def bootstrapDatabaseEngine(event):
    engine = component.getUtility(IDatabaseEngine)
    engine.metadata = metadata = rdb.MetaData(engine)
    setUp(metadata)
    
def setUp(metadata):
    soup(metadata)
    catalog(metadata)
    relation(metadata)
    metadata.create_all()

class Soup(object):
    pass

def soup(metadata):
    table = rdb.Table(
        'soup',
        metadata,
        rdb.Column('id', rdb.Integer, primary_key=True, autoincrement=True),
        rdb.Column('uuid', rdb.String(length=32), unique=True, index=True),
        rdb.Column('spec', rdb.String, index=True),
        )

    orm.mapper(Soup, table)

    return table

def catalog(metadata):
    return rdb.Table(
        'catalog',
        metadata,
        rdb.Column('id', rdb.Integer, primary_key=True, autoincrement=True),
        rdb.Column('left', rdb.String(length=32), rdb.ForeignKey("soup.uuid"), index=True),
        rdb.Column('right', rdb.String(length=32), rdb.ForeignKey("soup.uuid")),
        rdb.Column('name', rdb.String))

class Relation(object):
    def _get_source(self):
        return relations.lookup(self.left)

    def _set_source(self, item):
        self.left = item.uuid

    def _get_target(self):
        return relations.lookup(self.right)

    def _set_target(self, item):
        if not IMapped.providedBy(item):
            item = relations.persist(item)
        
        self.right = item.uuid

    source = property(_get_source, _set_source)
    target = property(_get_target, _set_target)
    
def relation(metadata):
    table = rdb.Table(
        'relation',
        metadata,
        rdb.Column('id', rdb.Integer, primary_key=True, autoincrement=True),
        rdb.Column('left', rdb.String(length=32), rdb.ForeignKey("soup.uuid"), index=True),
        rdb.Column('right', rdb.String(length=32), rdb.ForeignKey("soup.uuid")),
        rdb.Column('order', rdb.Integer, nullable=False))
    
    orm.mapper(Relation, table)
    
    return table



##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from setuptools import setup, find_packages

setup(name='z3c.dobbin',
      version='0.2.2',
      license='ZPL',
      author = "Malthe Borch, Stefan Eletzhofer and the Zope Community",
      author_email = "zope-dev@zope.org",
      description="Relational object persistance framework",
      long_description=open('README.txt').read()+open('src/z3c/dobbin/README.txt').read(),
      keywords='',
      classifiers=['Programming Language :: Python',
                   'Environment :: Web Environment',
                   'Framework :: Zope3',
                   ],
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['z3c'],
      include_package_data=True,
      zip_safe=False,
      extras_require = dict(
        test = [
            'zope.app.testing',
            ],
        ),
      install_requires = [ 'setuptools',
                           'zope.interface',
                           'zope.schema',
                           'zope.component',
                           'zope.dottedname',
                           'ore.alchemist',
                           'ZODB3',
                           'SQLAlchemy'],
      )
